package net.sf.sido.support;

import com.google.common.base.Function;
import net.sf.sido.DataObject;
import org.apache.commons.lang.ObjectUtils;

/**
 * Generator of functions.
 */
public class DOFunctions {

    /**
     * Function that extracts
     *
     * @param <X>
     */
    public static final class DOGetFunction<X> implements Function<DataObject, X> {

        private final String propertyName;

        public DOGetFunction(String propertyName) {
            this.propertyName = propertyName;
        }

        @Override
        @SuppressWarnings("unchecked")
        public X apply(DataObject input) {
            return (X) input.get(propertyName);
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            DOGetFunction that = (DOGetFunction) o;

            if (!propertyName.equals(that.propertyName)) return false;

            return true;
        }

        @Override
        public int hashCode() {
            return propertyName.hashCode();
        }

        @Override
        public String toString() {
            return "DOGetFunction{" +
                    "propertyName='" + propertyName + '\'' +
                    '}';
        }
    }

    /**
     * Function that extracts a property as a string
     */
    public static final class DOGetAsStringFunction implements Function<DataObject, String> {

        private final String propertyName;

        public DOGetAsStringFunction(String propertyName) {
            this.propertyName = propertyName;
        }

        @Override
        public String apply(DataObject input) {
            Object value = input.get(propertyName);
            return ObjectUtils.toString(value, null);
        }

    }

    /**
     * Creates a function that extracts a property from a SiDO instance.
     *
     * @param propertyName Property to get
     * @return Function
     */
    public static <X> Function<DataObject, X> get(String propertyName) {
        return new DOGetFunction<X>(propertyName);
    }

    /**
     * Creates a function that extracts a property from a SiDO instance as a string.
     *
     * @param propertyName Property to get
     * @return Function
     */
    public static Function<DataObject, String> getAsString(String propertyName) {
        return new DOGetAsStringFunction(propertyName);
    }

    /**
     * Cannot instantiate this class
     */
    private DOFunctions() {
    }
}
