package net.sf.sido.support;

import java.util.Map;

import net.sf.sido.DOInvalidPropertyDOTypeException;
import net.sf.sido.DataObject;
import net.sf.sido.DataType;
import net.sf.sido.ParentLink;
import net.sf.sido.Property;
import net.sf.sido.PropertyController;
import net.sf.sido.PropertyMetaInfos;

public class DOPropertyController implements PropertyController<DataObject> {

	private final DataType doType;
	private final boolean reverse;

	public DOPropertyController(DataType doType, boolean reverse) {
		this.doType = doType;
		this.reverse = reverse;
	}

	@Override
	public DataObject getDefaultValue(boolean deep) {
		if (deep) {
			return doType.newInstance(true, deep);
		} else {
			return null;
		}
	}

	@Override
	public void control(Property<DataObject> property, DataObject o, DataObject value) {
		// Checks the type
		if (!value.getSidoType().isSubClass(doType)) {
			throw new DOInvalidPropertyDOTypeException(o, property.getName(), doType, value.getSidoType());
		}
		// Parent relation-ship
		if (reverse) {
			value.setParentLink(new ParentLink(o, property.getName()));
		}
	}

	@Override
	public void fillMetaInfo(Map<String, Object> mi) {
		mi.put(PropertyMetaInfos.REVERSE, reverse);
		mi.put(PropertyMetaInfos.DO, true);
		mi.put(PropertyMetaInfos.DO_TYPE, doType);
	}

}
